@@ -21,10 +21,10 @@ from pynpoint.util.image import create_mask, scale_image, shift_image
 class PSFpreparationModule(ProcessingModule):
     """
     Module to prepare the data for PSF subtraction with PCA. The preparation steps include masking
-    and image normalization.
+    and an optional normalization.
     """
 
-    __author__ = 'Markus Bonse, Tomas Stolker, Timothy Gebhard'
+    __author__ = 'Markus Bonse, Tomas Stolker, Timothy Gebhard, Sven Kiefer'
 
     @typechecked
     def __init__(self,
@@ -49,7 +49,8 @@ class PSFpreparationModule(ProcessingModule):
             Tag of the database entry with the mask that is written as output. If set to None, no
             mask array is saved.
         norm : bool
-            Normalize each image by its Frobenius norm.
+            Normalize each image by its Frobenius norm. Only supported for 3D datasets (i.e.
+            regular imaging).
         resize : float, None
             DEPRECATED. This parameter is currently ignored by the module and will be removed in a
             future version of PynPoint.
@@ -66,7 +67,7 @@ class PSFpreparationModule(ProcessingModule):
             None
         """
 
-        super(PSFpreparationModule, self).__init__(name_in)
+        super().__init__(name_in)
 
         self.m_image_in_port = self.add_input_port(image_in_tag)
 
@@ -91,25 +92,40 @@ class PSFpreparationModule(ProcessingModule):
     def run(self) -> None:
         """
         Run method of the module. Masks and normalizes the images.
-
         Returns
         -------
         NoneType
             None
         """
 
-        # Get PIXSCALE and MEMORY attributes
+        # Get the PIXSCALE and MEMORY attributes
         pixscale = self.m_image_in_port.get_attribute('PIXSCALE')
         memory = self._m_config_port.get_attribute('MEMORY')
 
-        # Get the number of images and split into batches to comply with memory constraints
+        # Get the numnber of dimensions and shape
+        ndim = self.m_image_in_port.get_ndim()
         im_shape = self.m_image_in_port.get_shape()
-        nimages = im_shape[0]
-        frames = memory_frames(memory, nimages)
+
+        if ndim == 3:
+            # Number of images
+            nimages = im_shape[-3]
+
+            # Split into batches to comply with memory constraints
+            frames = memory_frames(memory, nimages)
+
+        elif ndim == 4:
+            # Process all wavelengths per exposure at once
+            frames = np.linspace(0, im_shape[-3], im_shape[-3]+1)
+
+        if self.m_norm and ndim == 4:
+            warnings.warn('The \'norm\' parameter does not support 4D datasets and will therefore '
+                          'be ignored.')
 
         # Convert m_cent_size and m_edge_size from arcseconds to pixels
+
         if self.m_cent_size is not None:
             self.m_cent_size /= pixscale
+
         if self.m_edge_size is not None:
             self.m_edge_size /= pixscale
 
@@ -121,34 +137,42 @@ class PSFpreparationModule(ProcessingModule):
         # we are not normalizing, this list will remain empty)
         norms = list()
 
-        # Run the PSFpreparationModule for each subset of frames
         start_time = time.time()
-        for i, _ in enumerate(frames[:-1]):
 
+        # Run the PSFpreparationModule for each subset of frames
+        for i in range(frames[:-1].size):
             # Print progress to command line
             progress(i, len(frames[:-1]), 'Preparing images for PSF subtraction...', start_time)
 
-            # Get the images and ensure they have the correct 3D shape with the following
-            # three dimensions: (batch_size, height, width)
-            images = self.m_image_in_port[frames[i]:frames[i+1], ]
+            if ndim == 3:
+                # Get the images and ensure they have the correct 3D shape with the following
+                # three dimensions: (batch_size, height, width)
+                images = self.m_image_in_port[frames[i]:frames[i+1], ]
 
-            if images.ndim == 2:
-                warnings.warn('The input data has 2 dimensions whereas 3 dimensions are required. '
-                              'An extra dimension has been added.')
+                if images.ndim == 2:
+                    warnings.warn('The input data has 2 dimensions whereas 3 dimensions are '
+                                  'required. An extra dimension has been added.')
 
-                images = images[np.newaxis, ...]
+                    images = images[np.newaxis, ...]
+
+            elif ndim == 4:
+                # Process all wavelengths per exposure at once
+                images = self.m_image_in_port[:, i, ]
 
             # Apply the mask, i.e., set all pixels to 0 where the mask is False
             images[:, ~mask] = 0.
 
             # If desired, normalize the images using the Frobenius norm
-            if self.m_norm:
+            if self.m_norm and ndim == 3:
                 im_norm = np.linalg.norm(images, ord='fro', axis=(1, 2))
                 images /= im_norm[:, np.newaxis, np.newaxis]
                 norms.append(im_norm)
 
             # Write processed images to output port
-            self.m_image_out_port.append(images, data_dim=3)
+            if ndim == 3:
+                self.m_image_out_port.append(images, data_dim=3)
+            elif ndim == 4:
+                self.m_image_out_port.append(images, data_dim=4)
 
         # Store information about mask
         if self.m_mask_out_port is not None:
@@ -170,6 +194,7 @@ class PSFpreparationModule(ProcessingModule):
             self.m_image_out_port.add_attribute(name='cent_size',
                                                 value=self.m_cent_size * pixscale,
                                                 static=True)
+
         if self.m_edge_size is not None:
             self.m_image_out_port.add_attribute(name='edge_size',
                                                 value=self.m_edge_size * pixscale,
@@ -202,7 +227,7 @@ class AngleInterpolationModule(ProcessingModule):
             None
         """
 
-        super(AngleInterpolationModule, self).__init__(name_in)
+        super().__init__(name_in)
 
         self.m_data_in_port = self.add_input_port(data_tag)
         self.m_data_out_port = self.add_output_port(data_tag)
@@ -264,7 +289,7 @@ class AngleInterpolationModule(ProcessingModule):
 
 class SortParangModule(ProcessingModule):
     """
-    Module to sort the images and non-static attributes with increasing INDEX.
+    Module to sort the images and attributes with increasing ``INDEX``.
     """
 
     __author__ = 'Tomas Stolker'
@@ -280,10 +305,10 @@ class SortParangModule(ProcessingModule):
         name_in : str
             Unique name of the module instance.
         image_in_tag : str
-            Tag of the database entry that is read as input.
+            Database tag with the input data.
         image_out_tag : str
-            Tag of the database entry with images that is written as output. Should be different
-            from *image_in_tag*.
+            Database tag where the output data will be stored. Should be different from
+            ``image_in_tag``.
 
         Returns
         -------
@@ -291,7 +316,7 @@ class SortParangModule(ProcessingModule):
             None
         """
 
-        super(SortParangModule, self).__init__(name_in)
+        super().__init__(name_in)
 
         self.m_image_in_port = self.add_input_port(image_in_tag)
         self.m_image_out_port = self.add_output_port(image_out_tag)
@@ -299,7 +324,8 @@ class SortParangModule(ProcessingModule):
     @typechecked
     def run(self) -> None:
         """
-        Run method of the module. Sorts the images and relevant non-static attributes.
+        Run method of the module. Sorts the images and attributes with increasing ``INDEX``.
+        Therefore, the images are sorted by there original (usually chronological) order.
 
         Returns
         -------
@@ -307,12 +333,12 @@ class SortParangModule(ProcessingModule):
             None
         """
 
-        if self.m_image_in_port.tag == self.m_image_out_port.tag:
-            raise ValueError('Input and output port should have a different tag.')
-
         memory = self._m_config_port.get_attribute('MEMORY')
         index = self.m_image_in_port.get_attribute('INDEX')
 
+        ndim = self.m_image_in_port.get_ndim()
+        nimages = self.m_image_in_port.get_shape()[-3]
+
         index_new = np.zeros(index.shape, dtype=np.int)
 
         if 'PARANG' in self.m_image_in_port.get_all_non_static_attributes():
@@ -331,11 +357,10 @@ class SortParangModule(ProcessingModule):
 
         index_sort = np.argsort(index)
 
-        nimages = self.m_image_in_port.get_shape()[0]
-
         frames = memory_frames(memory, nimages)
 
         start_time = time.time()
+
         for i, _ in enumerate(frames[:-1]):
             progress(i, len(frames[:-1]), 'Sorting images in time...', start_time)
 
@@ -347,9 +372,13 @@ class SortParangModule(ProcessingModule):
             if star_new is not None:
                 star_new[frames[i]:frames[i+1]] = star[index_sort[frames[i]:frames[i+1]]]
 
-            # h5py indexing elements must be in increasing order
-            for _, item in enumerate(index_sort[frames[i]:frames[i+1]]):
-                self.m_image_out_port.append(self.m_image_in_port[item, ], data_dim=3)
+            # HDF5 indexing elements must be in increasing order
+            for item in index_sort[frames[i]:frames[i+1]]:
+                if ndim == 3:
+                    self.m_image_out_port.append(self.m_image_in_port[item, ], data_dim=3)
+
+                elif ndim == 4:
+                    self.m_image_out_port.append(self.m_image_in_port[:, item, ], data_dim=4)
 
         self.m_image_out_port.copy_attributes(self.m_image_in_port)
         self.m_image_out_port.add_history('SortParangModule', 'sorted by INDEX')
@@ -394,7 +423,7 @@ class AngleCalculationModule(ProcessingModule):
             None
         """
 
-        super(AngleCalculationModule, self).__init__(name_in)
+        super().__init__(name_in)
 
         # Parameters
         self.m_instrument = instrument
@@ -609,7 +638,7 @@ class AngleCalculationModule(ProcessingModule):
 
 class SDIpreparationModule(ProcessingModule):
     """
-    Module for preparing continuum frames for SDI subtraction.
+    Module for preparing continuum frames for dual-band simultaneous differential imaging.
     """
 
     __author__ = 'Gabriele Cugno, Tomas Stolker'
@@ -644,7 +673,7 @@ class SDIpreparationModule(ProcessingModule):
             None
         """
 
-        super(SDIpreparationModule, self).__init__(name_in)
+        super().__init__(name_in)
 
         self.m_image_in_port = self.add_input_port(image_in_tag)
         self.m_image_out_port = self.add_output_port(image_out_tag)
@@ -675,7 +704,7 @@ class SDIpreparationModule(ProcessingModule):
 
         start_time = time.time()
         for i in range(nimages):
-            progress(i, nimages, 'Preparing images for SDI...', start_time)
+            progress(i, nimages, 'Preparing images for dual-band SDI...', start_time)
 
             image = self.m_image_in_port[i, ]
 
